#pragma once

#include "wtwPlugin.h"

////////////////////////////////////////////////////////////////////////
//
// Funkcje
//

//// WTW_NETLIB_CREATE
// zwraca zero przy bledzie, non-zero jesli poszlo ok
//
// wParam - netLibCreateDef*
//
#define WTW_NETLIB_CREATE  L"NetLib/Create"		

//// WTW_NETLIB_DESTROY
// zwraca zero
//
// wParam - const wchar_t * - ID stworzonego socketa
//
// koniecznie nalezy zwolnic wszystkie sockety przy wyladowaniu wtyczki
// lub wczesniej. jesli sie tego nie zrobi, ma sie 
// access volation jak w banku
//
#define WTW_NETLIB_DESTROY L"NetLib/Destroy"

//// WTW_NETLIB_CONNECT
// zwraca zero przy bledzie, non-zero jesli poszlo ok
//
// wParam - netLibConnectDef*
//
#define WTW_NETLIB_CONNECT L"NetLib/Connect"

//// WTW_NETLIB_CLOSE
// zwraca zero
//
// wParam - const wchar_t * - ID stworzonego socketa
//
#define WTW_NETLIB_CLOSE   L"NetLib/Close"

//// WTW_NETLIB_SEND
// zwraca zero przy bledzie, non-zero jesli poszlo ok
//
// wParam - netLibSendDef*
//
#define WTW_NETLIB_SEND	L"NetLib/Send"

//// WTW_NETLIB_INITSSL
// zwraca zero przy bledzie, non-zero jesli poszlo ok
//
// wParam - const wchar_t * - ID _poczonego_ socketa
//
#define WTW_NETLIB_INITSSL	L"NetLib/InitSSL"

////////////////////////////////////////////////////////////////////////
//
// Eventy
//

#define WTW_NETLIB_EVENT_DESTROY 0x0001
#define WTW_NETLIB_EVENT_CONNECT 0x0002
#define WTW_NETLIB_EVENT_CLOSE   0x0003
#define WTW_NETLIB_EVENT_DATA    0x0004
#define WTW_NETLIB_EVENT_ACCEPT  0x0005
#define WTW_NETLIB_EVENT_CLOSED  0x0006
#define WTW_NETLIB_EVENT_TIMEOUT 0x0007
#define WTW_NETLIB_EVENT_SSL	 0x0008
#define WTW_NETLIB_EVENT_ERROR	 0x0009 // to bedzie wywoane przy okazji "close" jesl nastpi bd poczenia

////////////////////////////////////////////////////////////////////////
//
// Opcje
//

#define WTW_NETLIB_PROXYTYPE_SOCKS4		0x01
#define WTW_NETLIB_PROXYTYPE_SOCKS4A	0x02
#define WTW_NETLIB_PROXYTYPE_SOCKS5		0x03
#define WTW_NETLIB_PROXYTYPE_HTTP11		0x04

#define WTW_NETLIB_FLAG_PROXY    0x00000001	// uzyc proxy
#define WTW_NETLIB_FLAG_SSL      0x00000002	// SSL, tylko ifnormacja, ze bedziemy go uzywac, aby uruchomic nalezy wywolac SSLINIT, jesli handshake sie powiedzie (lub nie) dostaniemy event _SSL 
#define WTW_NETLIB_FLAG_LISTEN   0x00000004	// Socket nasluchujacy
#define WTW_NETLIB_FLAG_TIMEOUT  0x00000008	// nie obslugiwane, timeout nastepuje automatycznie
#define WTW_NETLIB_FLAG_ACCEPT   0x00000010	// Utworz nowy socket i przyjmij poczenie, do tego musimy wypelnic fromSocket

////////////////////////////////////////////////////////////////////////
//
// Struktury
//

typedef struct {
	int				structSize;
	int				eventID;		// kod zdarzenia
	void		 *	sockAddr;		// wskaznik do CAsyncSocketEx 
	HANDLE			hSocket;		// handle do socketa
	const wchar_t*	iD;				// id socketa
	void		 *	ownerData;		// z netLibCreateDef;
} netLibEventDef;

typedef struct {
	int				structSize;
	const wchar_t*  hostAddr;		// adres serwera
	int				hostPort;		// port
	const wchar_t*	iD;				// id socketa

	const wchar_t *	proxyAddress;   // adres proxy
	int				proxyType;		// typ proxy
	int				proxyPort;		// port serwera proxy;

	const wchar_t *	proxyUser;		// wypelnic tylko jesli uzywamy autoryzacji, w innym wypadku - NULL
	const wchar_t *	proxyPass;		// j.w.

	int				flags;

} netLibConnectDef;

typedef struct {
	int				structSize;
	const char *	pBuffer;
	int				nBufferLen;
	const wchar_t *	iD;				// id socketa
} netLibSendDef;

typedef struct {
	int				structSize;
	
	const wchar_t *	bindAddress;	// adres IP (char*) pod ktory zbindowac polaczenie, mzoe byc NULL, wtedy zbindowany zostanie do 0.0.0.0
	int				bindPort;		// port, jesli podaje sie adres, port  musi byc wypelniony (musi tez byc dla polaczen przychodzacych)

	int				flags;			// opcje;
	int				timeout;		// czas w ms po ktorym nastepuje timeout (musi byc wybrany w opcjach)

	WTWFUNCTION		callBack;		// funkcja ktora bedzie otrzymywac informacje o zdarzeniach z socketa, jako wParam idzie netLibEventDef*, jako lParam w wypadki _DATA netLibDataDef* w innych - nULL

	void		*	fromSocket;		// Adres socketa z ktorego nadeszla informacja o nowym polaczeniu;

	const wchar_t *	iD;				// _jednoznaczny_ id po ktorym bedzie identyfikowany socket
	void		*	ownerData;		// chyba wiadomo ;)
} netLibCreateDef;

typedef struct {
	int				structSize;
	int				dataLen;		// dlugosc odebranych  danych
	void *			data;			// wskaznik do bufora
} netLibDataDef;

#define WTW_SSL_FAILURE_UNKNOWN		0
#define WTW_SSL_FAILURE_ESTABLISH	1
#define WTW_SSL_FAILURE_LOADDLLS	2
#define WTW_SSL_FAILURE_INITSSL		4 

typedef struct {
	int				structSize;
	int				sslStatus;		// 1 - OK, 0 - Blad
	int				sslInfo;		// WTW_SSL_FAIL...
} netLibSSLStatusDef;