#ifndef __wtw_plugin_settings_h__
#define __wtw_plugin_settings_h__

#include "wtwPlugin.h"
#include <wchar.h>

#pragma warning(disable:4311)
#pragma warning(disable:4312)
#pragma warning(disable:4267)

struct wtwMyConfigFile {
	int		structSize;
	wchar_t *pBuffer;
	int		bufferSize;
	const wchar_t * desiredExt; // moze byc NULL
};

struct wtwBuffer {
	int		structSize;
	wchar_t *pBuffer;
	int		bufferSize;			// w znakach!
};

/////////////////////////////////////////////////////////////////////////////////
//
// WTW_SETTINGS_GET_MY_CONFIG_FILE
// 
// wParam - wtwMyConfigFile* z wypelnionym pBuffer i bufferLen,
// lParam - hInstance wtyczki
// 
// do pBuffer zostanie skopiowana sciezka do pliku ustawien dla danej wtyczki
//

#define WTW_SETTINGS_GET_MY_CONFIG_FILE L"WTW/Settings/GetMyConfigFile"

/////////////////////////////////////////////////////////////////////////////////
//
// WTW_INFO_GET_CACHE_DIRECTORY
// 
// wParam - wtwBuffer* z wypelnionym pBuffer i bufferLen,
// lParam - NULL
// 
// do pBuffer zostanie skopiowana sciezka do folderu cache (nalezy sobie
// stworzyc odpowiedni podfolder)
//

#define WTW_INFO_GET_CACHE_DIRECTORY L"WTW/Info/GetCacheDir"

/////////////////////////////////////////////////////////////////////////////////
//
// WTW_INFO_GET_PROFILE_DIRECTORY
// 
// wParam - wtwBuffer* z wypelnionym pBuffer i bufferLen,
// lParam - NULL
// 
// do pBuffer zostanie skopiowana sciezka do folderu profilu
//

#define WTW_INFO_GET_PROFILE_DIRECTORY L"WTW/Info/GetProfileDir"

/////////////////////////////////////////////////////////////////////////////////
//
// WTW_SETTINGS_INIT
// 
// wParam - wchar_t * - sciezka do pliku
// lParam - NULL
// 
// zwraca (void *) -> pSettings
//

#define WTW_SETTINGS_INIT		L"WTW/Settings/Init"

/////////////////////////////////////////////////////////////////////////////////
//
// WTW_SETTINGS_DESTROY
// 
// wParam - (void *) pSettings
// lParam - NULL
// 
// jesli wolano init, to raczej nie nalezy zapomniec o destroy
//

#define WTW_SETTINGS_DESTROY	L"WTW/Settings/Destroy"

/////////////////////////////////////////////////////////////////////////////////
//
// WTW_SETTINGS_GET_VALUE
// 
// wParam - wtwSettingDef*
// lParam - NULL
// 
// zwraca 1 jesli znaleziono wartosc, 0 jesli nie, -1 buffer too small 
// (w takim wypadku paramValueLen bedzie zawieralo minimalny rozmiar bufora
// taki sam efekt (-1) uzyskamy ustawiajac paramValue na NULL)
//

#define WTW_SETTINGS_GET_VALUE	L"WTW/Settings/Get"

/////////////////////////////////////////////////////////////////////////////////
//
// WTW_SETTINGS_SET_VALUE
// 
// wParam - wtwSettingDef*
// lParam - NULL
// 
// 1 jesli ok, 0 przy bledzie
//

#define WTW_SETTINGS_SET_VALUE	L"WTW/Settings/Set"

/////////////////////////////////////////////////////////////////////////////////
//
// WTW_SETTINGS_READ
// 
// wParam - (void*)pSettings
// lParam - NULL
// 
// 1 jesli ok, 0 przy bledzie
//

#define WTW_SETTINGS_READ	L"WTW/Settings/Read"

/////////////////////////////////////////////////////////////////////////////////
//
// WTW_SETTINGS_WRITE
// 
// wParam - (void*) pSettings
// lParam - NULL
// 
// 1 jesli ok, 0 przy bledzie
//

#define WTW_SETTINGS_WRITE	L"WTW/Settings/Write"

/////////////////////////////////////////////////////////////////////////////////
//
// WTW_SETTINGS_SET_KEY
// 
// wParam - (void*) pSettings
// lParam - (char*) char[64]
// 
// 1 jesli ok, 0 przy bledzie - ustawia klucz szyfrowania dla pliku ustawien
// szyfrowane sa jedynei wartosci _STR (AES, 512 bit)
// jesli nie zostanie ustawiony zostanie uzyty domyslny
//

#define WTW_SETTINGS_SET_KEY	L"WTW/Settings/SetKey"

/////////////////////////////////////////////////////////////////////////////////
//
// 

struct wtwSettingDef {
	int				structSize;

	const wchar_t *	paramName;		//nazwa ustawienia, cos jak "WTW/ProxyAddres"

	int				paramValueType; // typ, int, sting, user
	void *			paramValue;		// wartosc
	int				paramValueSize;	// rozmiar takowej (tylko dla string i usr)

	void *			pSettings;		//to co zwrocilo INIT

};

#define WTW_SETTINGS_PVT_INT 0x01 // integer (int) == pvt
#define WTW_SETTINGS_PVT_STR 0x03 // string (wchar_t*) == pvt
#define WTW_SETTINGS_PVT_USR 0x02 // (void*) == pvt

int __inline wtwGetInt(
					   WTWFUNCTIONS *pF, 
					   void *pSett,
					   const wchar_t * sName, 
					   int defValue)
{
	wtwSettingDef sDef;
	initStruct(sDef);

	sDef.paramValueType = WTW_SETTINGS_PVT_INT;
	sDef.pSettings = pSett;
	sDef.paramName = sName;

	int ret = (int)pF->Call(WTW_SETTINGS_GET_VALUE, reinterpret_cast<TPARAM>(&sDef), NULL);

	if (ret)
		ret = (int)sDef.paramValue;
	else
		ret = defValue;

	return ret;
}

int __inline wtwSetInt(
					   WTWFUNCTIONS *pF, 
					   void *pSett,
					   const wchar_t * sName, 
					   int value)
{
	wtwSettingDef sDef;
	initStruct(sDef);

	sDef.paramValueType = WTW_SETTINGS_PVT_INT;
	sDef.pSettings = pSett;
	sDef.paramName = sName;
	sDef.paramValue= (void*)value;

	int ret = static_cast<int>(pF->Call(WTW_SETTINGS_SET_VALUE, reinterpret_cast<TPARAM>(&sDef), NULL));

	if (ret)
		ret = value;
	else
		ret = -1;

	return ret;
}

WTW_PTR __inline wtwGetStr(
					   WTWFUNCTIONS *pF, 
					   void *pSett,
					   const wchar_t * sName, 
					   const wchar_t * defValue,
					   wchar_t **value)
{
	wtwSettingDef sDef;
	initStruct(sDef);

	const int buffSize = 1024 * 4;

	sDef.paramValueType = WTW_SETTINGS_PVT_STR;
	sDef.pSettings = pSett;
	sDef.paramName = sName;

	sDef.paramValueSize = buffSize;
	sDef.paramValue     = (void*) new wchar_t [buffSize];
	
	WTW_PTR ret = pF->Call(WTW_SETTINGS_GET_VALUE, reinterpret_cast<TPARAM>(&sDef), NULL);

	if (!ret) {
		wcsncpy((wchar_t*)sDef.paramValue, defValue, buffSize - 1);
	}

	*value = (wchar_t*)sDef.paramValue;

	return ret; 
}

WTW_PTR __inline wtwSetStr(
					   WTWFUNCTIONS *pF, 
					   void *pSett,
					   const wchar_t * sName, 
					   const wchar_t * value)
{
	wtwSettingDef sDef;
	initStruct(sDef);

	sDef.paramValueType = WTW_SETTINGS_PVT_STR;
	sDef.pSettings = pSett;
	sDef.paramName = sName;

	sDef.paramValueSize = wcslen(value);
	sDef.paramValue     = (void*)value;
	
	return pF->Call(WTW_SETTINGS_SET_VALUE, reinterpret_cast<TPARAM>(&sDef), NULL);
}

/////////////////////////////////////////////////////////////////////////////////
//
// WTW_GET_PROFILE_NAME
// 
// wParam - wtwBuffer*
// lParam - NULL
// 
// zwraca dugosc nazwy profilu skopiowanej do bufora lub -1 jesli bufor byl za
// maly.
//

#define WTW_GET_PROFILE_NAME	L"WTW/Profile/GetName"


#endif