#ifndef __wtw__protocol_h__
#define __wtw__protocol_h__

#include "wtwPlugin.h"
#include "wtwCaps.h"
#include "wtwContact.h"

struct wtwMessageExtInfo {
	int				structSize;

	const wchar_t * name;
	const wchar_t * value;
};

struct wtwMessageDef {
	int				structSize;

	wtwContactDef	contactData;

	const wchar_t * msgTopic;
	const wchar_t * msgMessage;

	DWORD			msgFlags;
	time_t			msgTime;

	const wtwMessageExtInfo *extInfo;
	int				extInfoCount;

	wtwResourceInfo * exResource;

	void *			ownerData;

};

struct wtwAvatarDef{
	int				structSize;

	wtwContactDef	contactData;

	const wchar_t * avPath;
	DWORD			avType;
};

struct wtwAvatarEvent {
	int				structSize;

	const wchar_t * avFile;		// zlokalizowany w folderze profilu, wypowo _avatar.png

	const wchar_t *	sha1;
	const wchar_t *	md5;

	DWORD			event;
};

#define WTW_AVATAR_EVENT_DELETE		0x0001
#define WTW_AVATAR_EVENT_SETUP		0x0002

#define WTW_EVENT_AVATAR_CHANGED	L"WTW/OnAvatarChanged"

#define WTW_MESSAGE_FLAG_INCOMING	0x00000001
#define WTW_MESSAGE_FLAG_OUTGOING	0x00000002
#define WTW_MESSAGE_FLAG_CHAT_MSG	0x00000004
#define WTW_MESSAGE_FLAG_SPECIAL	0x00000008
#define WTW_MESSAGE_FLAG_CONFERENCE 0x00000010
#define WTW_MESSAGE_FLAG_CONF_CHAT  0x00000020
#define WTW_MESSAGE_FLAG_PICTURE	0x00000040
#define WTW_MESSAGE_FLAG_NOHTMLESC	0x00000080
#define WTW_MESSAGE_FLAG_WARNING	0x00000100
#define WTW_MESSAGE_FLAG_ERROR		0x00000200
#define WTW_MESSAGE_FLAG_INFO		0x00000400
#define WTW_MESSAGE_FLAG_NOTHEME	0x00000800
#define WTW_MESSAGE_FLAG_NOARCHIVE	0x00001000 // wymus brak zapisu do archiwum
#define WTW_MESSAGE_FLAG_MONOSPACE	0x00002000
#define WTW_MESSAGE_FLAG_FORCEDISP	0x00004000

struct wtwPresenceDef {
	int				structSize;

	wtwContactDef *	contactData;

	int				curStatus;		// obecny stan
	const wchar_t * curDescription;	// NULL - brak opisu
	time_t			curTimeStamp;	// obecny czas

	const void	  * exInfo;			// cos doczepione przez plugin protokou, prawidlowe tylko dla pierwszego przejscia
									// przez hook;

	wtwResourceInfo * exResource;

	DWORD			flags;
};

#define WTW_PRESENCE_FLAG_MENU			0x00000001	// wywolane menu
#define WTW_PRESENCE_FLAG_PREDEF		0x00000002	// predefiniowany opis
#define WTW_PRESENCE_FLAG_USER			0x00000004	// opis uzytkownika
#define WTW_PRESENCE_FLAG_PLUGIN		0x00000008	// wywolanie z pluga
#define WTW_PRESENCE_FLAG_EXIT			0x00000010	// komunikator jest zamykany
#define WTW_PRESENCE_FLAG_AUTO			0x00000020	// przy starcie automatyczne polaczenie
#define WTW_PRESENCE_FLAG_NO_POPUP		0x00000040	// nie generowac automatycznego popupa w tray'u
#define WTW_PRESENCE_FLAG_NO_ARSM		0x00000080	// wyczy automatyczny wybr stanu po zasobach (No Automated Resource Status Management)
#define WTW_PRESENCE_FLAG_AA_ENTER		0x10000000	// wejscie w tryb AA
#define WTW_PRESENCE_FLAG_AA_EXIT		0x20000000	// wyjscie z AA
#define WTW_PRESENCE_FLAG_LOCK_ENTER	0x40000000	// komputer zablokowany
#define WTW_PRESENCE_FLAG_LOCK_EXIT		0x80000000	// komputer odblokowany
#define WTW_PRESENCE_FLAG_SUSPEND		0x01000000	// komputer wchodzi w tryb uspienia, automagiczne rozlaczenie
#define WTW_PRESENCE_FLAG_RESET			0x02000000	// reset poczenia z jakiegos powodu
#define WTW_PRESENCE_FLAG_RESUME		0x04000000	// resume after suspend

#define WTW_PRESENCE_UNDEF		0xF000
#define WTW_PRESENCE_ONLINE		0x0001
#define WTW_PRESENCE_CHAT		0x0002
#define WTW_PRESENCE_DND		0x0003
#define WTW_PRESENCE_AWAY		0x0004
#define WTW_PRESENCE_XA			0x0005
#define WTW_PRESENCE_INV		0x0006
#define WTW_PRESENCE_OFFLINE	0x0007
#define WTW_PRESENCE_ERROR		0x0008
#define WTW_PRESENCE_BLOCK		0x0009

#define WTW_PRESENCE_OVL_NONE	0x0000	// brak nakladki
#define WTW_PRESENCE_OVL_DESC	0x0001	// ustawiony opis XMPPdesc.png
#define WTW_PRESENCE_OVL_HIDING	0x0002	// ukrywamy sie przed XMPPhiding.png
#define WTW_PRESENCE_OVL_HIDDEN	0x0003	// nie chcemy jego stanu XMPPhidden.png
#define WTW_PRESENCE_OVL_GOT_IP	0x0004	// mamy jego IP XMPPip.png
#define WTW_PRESENCE_OVL_NOL	0x0005	// nie ma ns na liscie XMPPnol.png

struct wtwRawDataDef {
	int				structSize;
	const char *	pData;
	int				pDataLen;
	DWORD			flags;
};

#define WTW_RAW_FLAG_TEXT	0x0001 // C-String, tj null terminated najlepiej jesli pData jest o 1 wieksze niz pDataLen (Wlasnie na null)
#define WTW_RAW_FLAG_BIN	0x0002 // dane binarne
#define WTW_RAW_FLAG_UTF	0x0004 // tekst zakodowany w UTF
#define WTW_RAW_FLAG_UNI	0x0008 // tekst, ale jako const wchar_t*
#define WTW_RAW_FLAG_OOB	0x0010 // OOB, Out Of band Data
#define WTW_RAW_FLAG_EXT	0x0020 // zewnetrzene dane, wstrzykniete - przez wtyczke na przyklad

struct wtwProtocolInterface {
	int			structSize;
	WTWFUNCTION pfAuthContact;	// wP - wtwContactDef, lP - WTW_AUTH_OPT...
	WTWFUNCTION pfDeleteContact;// wP - wtwContactDef
};

#define WTW_AUTH_OPTION_ADD_TO_LIST 0x00000001
#define WTW_AUTH_OPTION_AUTH_GRANT	0x00000002
#define WTW_AUTH_OPTION_NO_AUTH		0x00000004
#define WTW_AUTH_OPTION_CANCEL		0x00000008 // do nothing

#define WTW_PF_UPDATE_PROTO_INFO	L"WTW/PFSetInfo" // wP - protocolLoginInfo*

struct protocolLoginInfo {
	int				structSize;
	const wchar_t * loginID;
	const wchar_t * loginName;

	const wchar_t * protoID;
	int				protoSID;
};

WTW_PTR inline wtwSetProtoInfo(WTWFUNCTIONS *pF, 
							   const wchar_t * pID, int pSID, 
							   const wchar_t * lID, const wchar_t * lName)
{
	protocolLoginInfo pt;
	initStruct(pt);

	pt.loginID		= lID;
	pt.loginName	= lName;
	pt.protoID		= pID;
	pt.protoSID		= pSID;

	return pF->Call(WTW_PF_UPDATE_PROTO_INFO, (WTW_PARAM)&pt, 0);
}

struct wtwProtocolDef {
	int				structSize;

	const wchar_t * protoID;		// ID protokou, GG, TLEN, XMPP ...
	const wchar_t * protoName;		// nazwa
	const wchar_t * protoDescr;		// opis

	int				protoSID;		// second ID, to jest parametr _zwracany_ dzieki 
									// temu mozna miec wiele wtyczek o tym samym ID protokolu;
									// kontakty, etc, etc ... sa identyfikowane w/g sID i ID;

	const wchar_t * protoUUID;		// cos co generujemy i zapisujemy do konfiguracji,
									// tak aby nawet jesli SID sie zmieni mozna bylo
									// rozpoznac ten protokol po czyms;
									// powinno byc globalnie unikalne, z tym laczone sa informacje
									// dodatkowe, jak telefon, notka, adres, metakontakty, etc.

	DWORD			protoCaps1;		// to co potrafimy robic;
	DWORD			protoCaps2;		// to co potrafimy robic;
	DWORD			flags;

	wtwProtocolInterface pInterface;

	DWORD		  * protoState;		// wskaznik do DWORDa ktory trzyma informacje
									// o aktualnym stanie protokou;

};

#define WTW_PROTO_STATE_CONNECTED	0x00000001
#define WTW_PROTO_STATE_ENCRYPTED	0x00000002
#define WTW_PROTO_STATE_CONNECTING	0x00000004
#define WTW_PROTO_STATE_AUTH_OK		0x00000008

#define WTW_PROTO_FLAG_NO_MENU	0x0001	// nie dodawac menu stanw
#define WTW_PROTO_FLAG_PSEUDO	0x0002	// pseudo protokol, wtw bedzie olewal jego stany, etc ...

#define WTW_PROTO_FUNC_ADD		L"WTW/ProtocolAdd" // wParam = wtwProtocolDef*
#define WTW_PROTO_FUNC_DEL		L"WTW/ProtocolDel" // wP - pID, lP - SID

#define WTW_PROTO_EVENT_ON_ADD	L"WTW/OnProtocolAdd" // wParam = wtwProtocolDef*
#define WTW_PROTO_EVENT_ON_DEL	L"WTW/OnProtocolDel" // wP - pID, lP - SID

#define WTW_ON_PROTOCOL_EVENT	L"WTW/OnProtocolEvent" // to jest event kochani, hook it

struct wtwProtocolEvent {
	int				structSize;

	const wchar_t * pID;			// protocol ID
	int				pSID;			// protocol Second ID

	int				event;			// rodzaj zdarzenia
	int				type;			// typ

	wtwContactDef * ctData;			// jesli s dostepne sparsowane informacje o kontakcie inaczej NULL
};

#define	WTW_PEV_TYPE_BEFORE		0x0001 // przed przetworzeniem, mozna zablokowac zwracajac 1
#define	WTW_PEV_TYPE_AFTER		0x0002 // po przetworzeniu
#define	WTW_PEV_TYPE_UNDEFINED	0x0000 // zjawisko nie majace typu

#define WTW_PEV_RAW_DATA_RECV	0x0001
#define WTW_PEV_RAW_DATA_SEND	0x0002

#define WTW_PEV_PRESENCE_RECV	0x0003 // lParam wtwPresenceDef*
#define WTW_PEV_PRESENCE_SEND	0x0004 // lParam wtwPresenceDef*

#define WTW_PEV_MESSAGE_RECV	0x0005	// lparam wtwMessageGef*
#define WTW_PEV_MESSAGE_SEND	0x0006	// lparam wtwMessageGef*

#define WTW_PEV_NETWORK_LOGIN	0x0008
#define WTW_PEV_NETWORK_LOGOUT	0x0009

#define WTW_PEV_AUTH_REQUEST	0x000A
#define WTW_PEV_ROSTER_DLOADED	0x000B

#define WTW_PEV_XML_PROCESSED	0x0100	// it's blockable, it's internal, don't use ;)

#define WTW_PEV_ALERT_RECV			0x000D
#define WTW_PEV_TYPE_NOTIFY_RECV	0x000E
#define WTW_PEV_ALERT_SEND			0x000F
#define WTW_PEV_TYPE_NOTIFY_SEND	0x0010

#define WTW_PEV_AVATAR_RECV			0x0011
#define WTW_PEV_AVATAR_SEND			0x0012



HANDLE __inline wtwInstProtoFunc(
						WTWFUNCTIONS *pF,		// interface funkcji
						const wchar_t * pID,	// ID protokolu
						int				pSID,	// SID protokolu
						const wchar_t * funcName,// nazwa funkcji
						WTWFUNCTION		funcAddr // no wiadomo, ne ?
						)
{

	wchar_t fName[255] = {0};
	wsprintf(fName, L"%s/%d/%s", pID, pSID, funcName);
	return pF->Create(fName, funcAddr);

}

//////////////////////////////////////////////////////////////////////
//
// Nazwy chyba mowia same za siebie, jesli czegos nie mozemy zrobic
// bo na przyklad nie ma polaczenia to zwracamy 1, w innym wypadku
// powinno to byc 0 (jak nie to wtw poszuka innej wtyczki mogacej
// zrealizowac zadanie
//
// Te funkcje sa wywolywane przez komunikator, w wiekszosci;
//
// Postac tych funkcji to:
// ID/sID/Funkcja, instalujemy je uzywajac wtwInstProtoFunc
// ale nic nie stoi na przeszkodzie aby sobie robic to po swojemu,
// byle format byl prawidlowy
//
///

#define WTW_PF_STATUS_SET		L"PFStatusSet"
#define WTW_PF_STATUS_GET		L"PFStatusGet"
#define WTW_PF_MESSAGE_SEND		L"PFMessageSend"
#define WTW_PF_TN_SEND			L"PFTNSend"
#define WTW_PF_ALERT_SEND		L"PFAlertSend"

#define WTW_PF_RAW_DATA_SEND	L"PFRawSend"
#define WTW_PF_RAW_DATA_FEED	L"PFRawFeed"

#define WTW_PF_ASK_FOR_AUTH		L"WTW/PFAskForAuth" // wP - wtwContactDef*

#define WTW_PF_INIT_SEND_FILE	L"PFInitSendFile"
#define WTW_PF_SEND_PICT		L"PFSendPicture"
#define WTW_PF_SEND_IMGH		L"PFSendPictureByURL"



//////////////////////////////////////////////////////////////////////
//
// Jesli funkcja z tej grupy nie zostanie zarejestrowana, 
// i jesli plug zdacydowal, ze przykladowo pokazac odpowiednia
// pozycje w menu, to ww wywola domyslna implementacje
//
// wParam - wtwContactInfo *
// lParam - NULL
//
///

#define WTW_PF_INIT_CHAT		L"PFInitChat"
#define WTW_PF_INIT_MESSAGE		L"PFInitSendMessage"
#define WTW_PF_INIT_EDIT		L"PFInitEdit"
#define WTW_PF_INIT_ADD			L"PFInitAdd"
#define WTW_PF_INIT_SEND_PICT	L"PFInitSendPicture"


//////////////////////////////////////////////////////////////////////
//
// dlaczego funkcja a nie hookiem?
// bo hook i tak zostanei wywolany automatycznie, ale ta funkcja jest
// do czegos jeszcze, "karmi" ona komunikator danymi, jesli wiadomosc 
// nie zostanie odrzucona w TYPE_BEFORE to w TYPE_AFTER zostanie
// przetrawiona przez wtw (czyli wyswietlona, etc ...)
//
// wParam - wtwProtoEventDef *
// lParam - w zaleznosci co to za event ...
//
///

#define WTW_PF_CALL_HOOKS		L"WTW/CallProtolHooks" //

//////////////////////////////////////////////////////////////////////
//
// W ramach informacji, jesli chcemy ustawic globalnie cos dla
// wszystkich protokolow to mozemy zrobic tu
//
///

#define WTW_PF_WTW_STATUS_GET	L"WTW/PFStatusGet"
#define WTW_PF_WTW_STATUS_SET	L"WTW/PFStatusSet"
#define WTW_PF_WTW_SETUP_TN		L"WTW/PFTNSet"			// wP - wtwContactDef, lP: 0 cancel, 1 set
														// autocancel po 20 sec

#define WTW_PF_WTW_SEND_MESSAGE L"WTW/PFSendMessage"	// tutaj w contactDef musimy podac UUID kontaktu!
														// UUID! WTW sam zdecyduje ktorym protokolem to poslac
														// jesli chcemy konkretny protokol to uzywamy 
														// ID/sID/PFMessageSend i tam podajemy contactID!


#define WTW_PF_WTW_SHOW_MESSAGE		L"WTW/PFShowMessage"	// wPara - wtwMessageDef*

#define WTW_PF_GET_PROTOCOL_COUNT	L"WTW/PFGetCount"

// wewntrzna funkcja ;p

#define WTW_PF_GET_PROTOCOL_IPTR	L"WTW/PFGetIPtr"


#endif